; This is the uSCOPE program source for the 12F675 PIC processor.
;
; copy right: Ronald Dekker 2006
;
; It assembles error free with Microchip MPASM 03.20.02 assembler
; running under DOS
;
        LIST            P=PIC12f675
        ERRORLEVEL      -302    ;SUPPRESS BANK SELECTION MESSAGES
;        __CONFIG        01192H   ;XT OSC, WATCHDOG
        __config        _HS_OSC & _PWRTE_ON & _WDT_OFF & _CP_OFF & _BODEN_ON  & _MCLRE_OFF
;
INCLUDE P12F675.INC
;
;THE CONSTANTS
;
BEGIN_MEM	EQU	02DH	;begin byte of sample memory
DEBOUNCE	EQU	008H	;reload value of debounce counter 0..F
;    
;
; define the bits in GPIO
SYNC_B		EQU	2
VID_B		EQU	0
INP_B		EQU	3
; define the bits in BITS1
NINP		EQU	0
LAST		EQU	1
SINGLE		EQU	2
SAMPLE		EQU	3
ALWAYS		EQU	4
PAUSE		EQU	5
;
; define the bytes
BITS1		EQU	020H	;..,..,PAUSE,ALWAYS,SAMPLE,SINGLE,LAST,NINP
BITS2		EQU	021H	;used for key info and debounce
CNT_1		EQU	022H	;counter for synchronous part
CNT_2		EQU	023H	;counter for synchronous part
CNT_3		EQU	024H	;counter for asynchronous part
FR_CNT		EQU	025H	;frame counter
N_LINE		EQU	00EH	;line counter in a block
OFFSET		EQU	00FH	;offset for storing data
TIMEBASE	EQU	026H	;timebase counter
TB_RELOAD	EQU	027H	;reload value for timebase
SAVE_S		EQU	028H	;save of status during interrupt
SAVE_W		EQU	029H	;save of working register during interrupt
LEVEL_IND	EQU	02AH	;trigger level indicator
ADR_EE		EQU	02BH
ADR_TAB		EQU	02CH
;
;
		org	0000H
		goto	init
;
		org	0004H
		goto	sync_blck
;
; initialize special function registers (machine parameters)
init:		bsf	STATUS,RP0	;bank 1
		movlw	032H		;only AN1 is input
		movwf	TRISIO		;
;		movlw	022H		;clock/32, AN1 is analog
;		movlw	b'00100101'	; clock/16
;		movlw	b'00100001'	; clock/8
;		movlw	b'00100100'	; clock/4
		movlw	b'00100000'	; clock/2
		movwf	ANSEL		;
		movlw	089H		;VREF=on,low range, Vref=2.5V
		movwf	VRCON		;set reference voltage
		movlw	0D8H		;tmr0 internal clock, no prescaler
		movwf	OPTION_REG	;	
		bcf	STATUS,RP0	;bank 0
		clrf 	GPIO		;clear GPIO
		movlw	005H		;init ADCON0, left just./VDD=ref
		movwf	ADCON0		;AN1=input/converter on
		movlw	014H		;CINV=1,CIS=0,conf=4 (GP1=-,VREF=+)
		movwf	CMCON		;set comparator
; initialize general purpose registers (program parameters)
		movlw	BEGIN_MEM	;init FSR (first location for data)
		movwf	FSR		;	
;		movlw	080H		;dummy result of first AD conversion
;		movwf	ADRESH		;
		clrf	BITS1		;clear ALWAYS,SINGLE,SAMPLE
		movlw	002H		;timebase per default to 1.6 ms
		movwf	TB_RELOAD	;	
		movwf	TIMEBASE	;	
		movlw	088H		;trigger level indicator to 
		movwf	LEVEL_IND	;correspond to VRCON (see above)
;		movlw	050H		;number of lines in async lines block
;		movwf	N_LINE		;
		movlw	003H		;tables are in the upper 256 bytes
		movwf	PCLATH		;
; start the display:
; This call starts the synchronous subroutine. 
; Its function of this routine is to generate the video signal, 
; to sample and store the scope input and to sense the keys.
; A special part of the display area, located in the upper left corner 
; of the display is used to perform asynchronous tasks. In between two AD
; samples the subroutine returns to the asynchronous main program below.
; before its return, it sets timer0 to generate an interrupt to be back in
; time for the next AD sample.
;
		call 	sync_blck	;
;
; when we return from this routine, the routine has set timer0 so that
; when timer0 overflows, we automatically, through the interrupt vector, 
; return to the routine.	
;
; In the first block of the main program we first check if we need to retrieve
; settings from the eeprom and we build the display. If AA has been written
; in eeprom location 46H, valid settings are present and they are read and
; stored in the appropriate registers. Since for writing of the settings
; to the eeprom the same routines were used as for writing the display bits,
; the setting values are also stored at 7 bytes intervals, but the read routine
; is adapted to that. Next the display is built. Since the first two 
; characters depend on the data in the registers (either initialized of 
; restored) a bit of tricky program is used to sort them out.
;
		movlw	046H		;point to first databyte in eeprom
		movwf	ADR_EE		;
		call	read_ee		;get that data (valid flag)
		xorlw	0AAH		;is it valid (=AA)
		btfss	STATUS,Z	;
		goto	init_1		;jump if not valid, if valid continue
					;
		call	read_ee		;so the flag indicated that valid
		movwf	CMCON		;data is stored in the eeprom.
		call	read_ee		;In this block this data is restored 
		movwf	BITS1		;into the relevant registers
		call	read_ee		;
		bsf	STATUS,RP0	;VRCON is in bank 1
		movwf	VRCON		;
		bcf	STATUS,RP0	;


		call	read_ee		;
		movwf	LEVEL_IND	;
		call	read_ee		;
		movwf	TB_RELOAD	;
		movwf	TIMEBASE	;
					;in the next block the display is built
					;The characters that are not fixed are
					;derived from the relevant registers
					;(either initialized of restored)
init_1:		movlw	000H		;point to first character
		movwf	ADR_EE		;
		btfsc	BITS1,ALWAYS	;find out first character
		goto	init_2		;
		btfss	CMCON,CINV	;
		goto	init_3		;
		movlw	085H		;neg-to-pos
		goto	init_4		;
init_2:		movlw	077H		;no trigger detect
		goto	init_4		;
init_3:		movlw	07EH		;pos-to-neg
init_4:		call 	write_char	;
					;
		btfsc	BITS1,PAUSE	;find out second character
		goto	init_5		;
		btfsc	BITS1,SINGLE	;
		goto	init_6		;
		movlw	062H		;normal
		goto	init_7		;
init_5:		movlw	054H		;pause/skip
		goto	init_7		;
init_6:		movlw	05BH		;hold
init_7:		call 	write_char
					;the rest of the characters
		movlw	046H		;"L"
		call 	write_char	;
		movlw	070H		;"S"
		call 	write_char	;
		call	set_TB		;the two timebase digits
		movlw	069H		;"m"
		call 	write_char	;
;
; The next part constitutes the "body" of the main program. It is a sequence
; of five blocks. Each block deals with a specific function: 1. trigger level
; selection, 2. run mode, 3. trigger level, 4.store settings, 5. time-base 
; adjust. Each time the select button is pushed, the program rotates to the 
; next block. The select routine at the beginning takes care of the switching 
; on and off of the cursor. It also samples the key flags in BITS2
;
					;
					;trigger mode block
bl1:		movlw	03FH		;location of the cursor
		call	select		;
		btfsc	STATUS,C	;if C set next block
		goto	bl2		;
		clrf	ADR_EE		;point to first character
		btfsc	BITS1,ALWAYS	;are we in state 1 ?
		goto	bl1_2		;it was state 1 and becomes state 2
		btfss	CMCON,CINV	;are we in state 3 ?
		goto	bl1_1		;it was state 3 and becomes state 1
		bcf	CMCON,CINV	;so it was state 2 and it becomes 3
		movlw	07EH		;symbol high-to-low trigger
		goto	bl1_3		;display char and end
bl1_1:		bsf	BITS1,ALWAYS	;goto state 1
		bsf	CMCON,CINV	;
		movlw	077H		;symbol no trigger
		goto	bl1_3		;display char end end
bl1_2:		bcf	BITS1,ALWAYS	;goto state 2
		movlw	085H		;symbol low-to-high trigger
bl1_3:		call	write_char	;display the character		
		goto	bl1		;to the beginning of the block
					;
					;run mode block
bl2:		movlw	040H		;location of the cursor
		call	select		;
		btfsc	STATUS,C	;if C set next block
		goto	bl3		;
		movlw	001H		;point ADR_EE to second character
		movwf	ADR_EE		;
		btfsc	BITS1,PAUSE	;are we in state 2 ?
		goto	bl2_1		;it was state 2 and becomes state 3
		btfsc	BITS1,SINGLE	;are we in state 3 ?
		goto	bl2_2		;it was state 3 and becomes state 1
		bsf	BITS1,PAUSE	;go from state 1 to state 2
		movlw	054H		;load pause character 
		goto	bl2_3		;
bl2_1:		bcf	BITS1,PAUSE	;goto state 3
		bsf	BITS1,SINGLE	;
		movlw	05BH		;load hold character
		goto	bl2_3		;
bl2_2:		bcf	BITS1,SINGLE	;goto state 1
		movlw	062H		;load run character
bl2_3:		call	write_char	;write the character
		goto	bl2		;loop this block
					;
					;trigger adjust block
bl3:		movlw	041H		;location of the cursor
		call	select		;
		btfsc	STATUS,C	;if C set next block
		goto	bl4		;
		bsf	STATUS,RP0	;bank 1 because of VRCON
		clrf	CNT_3		;clear counter
bl3_1:		movf	CNT_3,0		;copy counter to w
		call	tr_tab		;do we have a VRCON value ?
		xorwf	VRCON,0		;compare table value with VRCON
		incfsz	CNT_3,1		;increment the counter, we use this
					;instruction here because it does not
					;affect Z and 0 is never reached
		btfss	STATUS,Z	;if z=1, w=0
		goto	bl3_1		;loop for next table entry
		btfss	BITS2,4		;see if we had to decrement counter
		goto	bl3_2		;no
		decf	CNT_3,1		;decrement counter by 2 (it was already
		decf	CNT_3,1		;incremented by 1!)
bl3_2:		movlw	007H		;use only lower 3 bits (only 8 values)
		andwf	CNT_3,1		;
		movf	CNT_3,0		;copy counter to w
		call	tr_tab		;get new VRCON value
		movwf	VRCON		;store it
		movlw	008H		;go to upper part of table
		addwf	CNT_3,0		;add 8 to counter and leave it in w
		call	tr_tab		;get trigger display level value
		movwf	LEVEL_IND	;store it
		bcf	STATUS,RP0	;bank 0 again
		goto	bl3		;do this block again
					;
					;store block
bl4:		movlw	042H		;location of the cursor
		call	select		;
		btfsc	STATUS,C	;if C set next block
		goto	bl5		;
		movlw	046H		;first address in eeprom is 46
		movwf	ADR_EE		;
		movlw	0AAH		;write AA to show that there is 
		call	write_ee	;data in eeprom
		movf	CMCON,0		;store the registers (note that 
		call	write_ee	;the address in incremented by
		movf	BITS1,0		;the routine write_ee)
		call	write_ee	;
		bsf	STATUS,RP0	;VRCON is in bank 1
		movf	VRCON,0		;
		bcf	STATUS,RP0	;
		call	write_ee	;
		movf	LEVEL_IND,0	;
		call	write_ee	;
		movf	TB_RELOAD,0	;
		call	write_ee	;
		goto	bl4
					;
					;timebase adjust block
bl5:		movlw	045H		;location of the cursor
		call	select		;
		btfsc	STATUS,C	;if C set next block
		goto	bl1		;
		btfss	BITS2,4		;up ("4"=0) or down ("4"=1)  
		goto	bl5_1		;go and do up
		rlf	TB_RELOAD,1	;it was down TB=TB*2
		btfsc	STATUS,C	;was bit in C?
		rrf	TB_RELOAD,1	;yes, shift it back
		goto	bl5_2		;finished doing down
bl5_1:		rrf	TB_RELOAD,1	;it was down TB=TB/2
		btfsc	STATUS,C	;was bit in C?
		rlf	TB_RELOAD,1	;yes, shift it back
bl5_2:		call	set_TB		;finished up, update display
		goto	bl5		;do this block again
;
; start scanning the frames
; Each frame on the TV is sub-divided into blocks. Each block displays a number
; of lines of one type. For example it can be that there is a block which
; displays 10 blank lines, or there is a block which displays 3 vertical sync.
; lines. In the program below this is realized. Each block of code
; represents a block of lines on the screen. First the number of lines of a 
; certain type is loaded, then the subroutine associated with that line
; is called that number of times. The program then continues to the next block
; until finishes. The programming of the code looks a bit strange, but in this
; way it takes exactly the same amount of time for a block to loop or to skip
; to the next block. The overhead generated by this "dispatcher" is exactly
; 8 machine cycles. 
;
;start block_1
;3 lines that generate the vertical sync pulse and sample the keys
		movlw	003H		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_1		;start first line
bloop_1:	nop			;add delay for all other lines
		goto	block_1	
block_1:	call	line_sync	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_1		;another one if not finished
;
; start block_2						
; 76 black lines with an asynchronous part between the AD sample 1 and 2
; Note that here the subroutine is not called, but that a simple jump is made.
; The reason is that in Line_asyn we temporarily return to the asynchronous
; main program. On an interrupt by timer 0 we then return to line_asyn. This 
; is not possible if line_asyn should be called at that time
		movlw	04CH		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_2		;start first line
bloop_2:	nop			;add delay for all other lines
		goto	block_2	
block_2:	goto	line_asyn	;do the line
block_2_ret:	decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_2		;another one if not finished
;
; start block_3						
; 2 lines that form the marker ticks at the upper part of the scope screen
		movlw	002H		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_3		;start first line
bloop_3:	nop			;add delay for all other lines
		goto	block_3	
block_3:	call	line_mark	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_3		;another one if not finished
;
; start block_4						
; 1 line that generates the horizontal upper part of the scope frame
		movlw	001H		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_4		;start first line
bloop_4:	nop			;add delay for all other lines
		goto	block_4	
block_4:	call	line_frame	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_4		;another one if not finished
;
; start block_5
; 64 lines which build the part of the scope image above the grid line
		movlw	040H		;number of lines in block_3
		movwf	N_LINE		;
		goto	block_5		;start first line
bloop_5:	nop			;add delay for all other lines
		goto	block_5	
block_5:	call	line_view	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_5		;another one if not finished
;
; start block_6
; 1 line which is the center grid line
		movlw	001H		;number of lines in block_3
		movwf	N_LINE		;
		goto	block_6		;start first line
bloop_6:	nop			;add delay for all other lines
		goto	block_6	
block_6:	call	line_mark	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_6		;another one if not finished
;
; start block_7
; 64 lines which build the part of the scope image below the grid line
		movlw	040H		;number of lines in block_3
		movwf	N_LINE		;
		goto	block_7		;start first line
bloop_7:	nop			;add delay for all other lines
		goto	block_7	
block_7:	call	line_view	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_7		;another one if not finished
;
; start block_8
; 1 line that generates the horizontal lower part of the scope frame
		movlw	001H		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_8		;start first line
bloop_8:	nop			;add delay for all other lines
		goto	block_8	
block_8:	call	line_frame	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_8		;another one if not finished
;
; start block_9
; 2 lines that form the marker ticks at the lower part of the scope screen
		movlw	002H		;number of lines in block_1
		movwf	N_LINE		;
		goto	block_9		;start first line
bloop_9:	nop			;add delay for all other lines
		goto	block_9	
block_9:	call	line_mark	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_9		;another one if not finished
;
; start block_10						
; 18 black lines
		movlw	012H		;number of lines in block_6
		movwf	N_LINE		;
		goto	block_10	;start first line
bloop_10:	nop			;add delay for all other lines
		goto	block_10	
block_10:	call	line_blank	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_10	;another one if not finished
;
; start block_11
; 10 lines for text (7 for character, 2 black spacing, 1 cursor)	
		movlw	00AH		;number of lines in block_4
		movwf	N_LINE		;
		goto	block_11	;start first line
bloop_11:	nop			;add delay for all other lines
		goto	block_11	
block_11:	call	line_txt	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_11	;another one if not finished
;
; start block_12
; 70 black lines that complete the frame					
		movlw	046H		;number of lines in block_4
		movwf	N_LINE		;
		goto	block_12	;start first line
bloop_12:	nop			;add delay for all other lines
		goto	block_12
block_12:	call	line_blank	;do the line
		decfsz	N_LINE,1	;decrement N_LINE
		goto	bloop_12	;another one if not finished
;
; in total we have now written 3+76+2+1+64+1+64+1+2+18+10+70=312 lines
; we are finished. What follows now is a nasty bit of programming.
; At the end of each frame (here) a frame counter is incremented (by 2)
; the frame counter is used to implement the skip/pause run mode (activated
; if PAUSE is true. If PAUSE is set then the image is frozen if FR_CNT > 32
; (upper 7 bits .neq. 000). At the end we reload for block_1 again 
; and jump back to the first block.
		movlw	003H		;number of lines in block_1	
		movwf	N_LINE		;
		btfss	BITS1,PAUSE	;if we are not in pause mode 
		goto	block_1		;goto first block again
		incf	FR_CNT		;FR_CNT=FR_CNT+2
		incf	FR_CNT		;
		movlw	0E0H		;is one of the upper 3 bits of
		andwf	FR_CNT,0	;FR_CNT set ?
		bsf	BITS1,SINGLE	;if so freeze the image if not 
		btfss	STATUS,Z	;do acquire data
		goto 	block_1		;
		bcf	BITS1,SINGLE	;
		goto	block_1
;		
; This routine displays one line of the actual scope image. The sampled
; data is stored in the upper part of the ram. we only need 128 datapoints
; to make an acceptable image on the screen. We have chosen to use only the
; upper 7 bits of each sample. All sample values are therefore even numbers.
; This routine works by successively incrementing each memory location
; by 2. Each time an overflow occurres, the carry bit is set. At that moment
; the corresponding bit on the screen is switched on by shifting the carry
; bit into the output register. Note that during this shifting operation
; no spurious sync pulses should be generated. This is achieved by setting
; the sync output bit to input during the critical part of the program.
; The add and shift procedure is so fast that a nop needed to be added to
; keep the image in a acceptable proportion.
line_view:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;[31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]
;		movlw	015H		;[69]
;		call	delay		;
						; 2 bytes gained 
;		call	getAD		; 31
;temporaily moved below
		call	getAD		; 31 [62 total] , 7 cycles ahead. . 2 bytes lost 
 
		bsf	STATUS,RP0      ;make sync_bit input [4]
		movlw	036H		;
		movwf	TRISIO		;
		bcf	STATUS,RP0      ;
		call	getAD		;[31]
		call	getAD		;[31], [minus 7 ahead cycles gained in removing delay, 
					; we have exact 24 extra cycles required to reach 64us with 28.496 crystal. 
				

		movlw	002H		;load increment in w [1]
		addwf	LEVEL_IND,1	;[1]
		rlf	GPIO		;[1]
		nop			;[1]
 		addwf	02EH,1		;50*[3]=[150] total
		bsf	GPIO,0		;switch frame on
		rlf	GPIO
		nop
		addwf	02FH,1		;
		rlf	GPIO
		nop
		addwf	030H,1		;
		rlf	GPIO
		nop
		addwf	031H,1		;
		rlf	GPIO
		nop
		addwf	032H,1		;
		rlf	GPIO
		nop
		addwf	033H,1		;
		rlf	GPIO
		nop
		addwf	034H,1		;
		rlf	GPIO
		nop
		addwf	035H,1		;
		rlf	GPIO
		nop
		addwf	036H,1		;
		rlf	GPIO
		nop
		addwf	037H,1		;
		rlf	GPIO
		nop
		addwf	038H,1		;
		rlf	GPIO
		nop
		addwf	039H,1		;
		rlf	GPIO
		nop
		addwf	03AH,1		;
		rlf	GPIO
		nop
		addwf	03BH,1		;
		rlf	GPIO
		nop
		addwf	03CH,1		;
		rlf	GPIO
		nop
		addwf	03DH,1		;
		rlf	GPIO
		nop
		addwf	03EH,1		;
		rlf	GPIO
		nop
		addwf	03FH,1		;
		rlf	GPIO
		nop
		addwf	040H,1		;
		rlf	GPIO
		nop
		addwf	041H,1		;
		rlf	GPIO
		nop
		addwf	042H,1		;
		rlf	GPIO
		nop
		addwf	043H,1		;
		rlf	GPIO
		nop
		addwf	044H,1		;
		rlf	GPIO
		nop
		addwf	045H,1		;
		rlf	GPIO
		nop
		addwf	046H,1		;
		rlf	GPIO
		nop
		addwf	047H,1		;
		rlf	GPIO
		nop
		addwf	048H,1		;
		rlf	GPIO
		nop
		addwf	049H,1		;
		rlf	GPIO
		nop
		addwf	04AH,1		;
		rlf	GPIO
		nop
		addwf	04BH,1		;
		rlf	GPIO
;		nop
		bsf	ADCON0,GO			; changed this to sample in middle of the line...
		addwf	04CH,1		;
		rlf	GPIO
		nop
		addwf	04DH,1		;
		rlf	GPIO
		nop
		addwf	04EH,1		;
		rlf	GPIO
		nop
		addwf	04FH,1		;
		rlf	GPIO
		nop
		addwf	050H,1		;
		rlf	GPIO
		nop
		addwf	051H,1		;
		rlf	GPIO
		nop
		addwf	052H,1		;
		rlf	GPIO
		nop
		addwf	053H,1		;
		rlf	GPIO
		nop
		addwf	054H,1		;
		rlf	GPIO
		nop
		addwf	055H,1		;
		rlf	GPIO
		nop
		addwf	056H,1		;
		rlf	GPIO
		nop
		addwf	057H,1		;
		rlf	GPIO
		nop
		addwf	058H,1		;
		rlf	GPIO
		nop
		addwf	059H,1		;
		rlf	GPIO
		nop
		addwf	05AH,1		;
		rlf	GPIO
		nop
		addwf	05BH,1		;
		rlf	GPIO
		nop
		addwf	05CH,1		;
		rlf	GPIO
		nop
		addwf	05DH,1		;
		rlf	GPIO
		nop
		addwf	05EH,1		;
		rlf	GPIO
		nop
		addwf	05FH,1		;
		rlf	GPIO
					;a nop has been deleted here
		addwf	OFFSET		;increment OFFSET as well [1]
		bsf	GPIO,0		;output on to end frame [1]

		bcf	GPIO,0		;output off [1]
		call	getAD		;the delayed acquisition [31]
		call	getAD		;[31]
		bsf	STATUS,RP0	;other outputs again on [4]
		movlw	032H
		movwf	TRISIO
		bcf	STATUS,RP0
;		movlw	013H		;finish up the remaining time
;		call	delay		; [62]
		call	getAD
		call	getAD		; get 2 samples instead of delay. exact same time...
;		nop			; [1]	; one nop had to be removed. no memory :(

		call	getAD
		; this was on the very top

		return			; [2] + [8] overhead
					; makes a total of [432] = 64us	
	; with 28.496 crystal, single instruction is 140.37058 ns long. 
	; so 64us , 64000ns , requires 455.93599 instructions.
	; this gives us 24 extra instructions.
	; instead of one 69 cycle delay, we did 2 AD acquisitions
	; so had 7 cycle gain. then added another AD acquisition, 31 cycles long.
	; this , minus 7 cycle gain, provides exactly 24 extra instructions.
	

	; THIS PART IS FIXED. there are 3+2 = 5 extra acquisitions. 


;
; This line routine generates one simple black line. Like in all lines,
; four samples are taken during the line. The first sample exactly fits into
; the sync pulse at the beginning of the line.
line_blank:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]
;		movlw	017H		;delay [74]
;		call	delay		;
;		nop			;[1]
					; instead of 75 cycles
		call 	getAD		; [31] 
		call	getAD		; [31]
					; 62 cycles. 13 cycles ahead.
					; there is 24 cycles extra due to faster clock.
					; so total 37 cycles ahead.  
					; there is extra acquisition @ the end
					; so there is still 37-31 , 6 cycles left!
					; 
					; it is 6 cycles ahead
					; 
 
					; we need to delay 6  cycles. tweaking delays... :

		call	getAD		;[31]

;		movlw	018H		;delay [77]
		movlw	d'25'		; delay [89] . 80-77 = 3 loss. 

		call 	delay		;
		call	getAD		;[31]
;		movlw	018H		;delay [77]
		movlw	d'26'		; delay [89]. 83-77 = 9 loss. total 6 loss... but... 

		call	delay		;
		call	getAD		;[31]

;		movlw	014H		;delay [65]

;		movlw	d'19'		; delay [62]. 65-62= 3. gain. . so total 9 loss
;		call	delay		;

		call	getAD		; [31]
		call	getAD		; [31] [total 62]. same as delay. and two extra acquisitions!

		call	getAD		; extra [31] cycles. 

		goto	line_blank_1	;[2]
line_blank_1:	return			; [2] + [8] overhead
					; makes a total of [432] = 64us

	; with 28.496 crystal, single instruction is 140.37058 ns long. 
	; so 64us , 64000ns , requires 455.93599 instructions.

	; this gives us 24 extra instructions. we removed one 75 cycle delay
	; added 62 cycles and 2 AD acquisitions. 13 cycles ahead. another acquisition
	; was added to the end. this steals 31 cycles. 
	; with the extra 24 instructions, this gives us 5 cycles short of time.
	; we gained this time by reducing leghts of delays. first 12 cycles, then 3 and then another 3 . 

	; last delay is replaced with two acquisitions. 

	; THIS PART IS FIXED . 5 extra acquisitions. 


;
; This line routine generates the solid white lines which are on the top
; and on the bottom of the display area. At the end of this routine the 
; eeprom address is cleared so that when the text lines start we start at the
; beginning of the eeprom memory.
line_frame:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]
;		movlw	017H		;delay [74]
;		call	delay		;
		call	getAD		; [31]
		call	getAD		; [31] [62 total], 74-62 = 12 ahead


;		nop			;[1]	
					; 13 ahead... we are saving memory

		call	getAD		;[31]

		goto 	line_frame_1	;[2]
line_frame_1:	
		
		bsf	GPIO,0		;line on [1]
					; we are 13 cycles ahead of where line SHOULD be. 		
					; actually just 6. 

;		movlw	017H		;delay [74]
;		call 	delay		;
	
		call	getAD		; [31]
		

		call	getAD		; [31]. [62 total] 74-62= 12 ahead. 
					; 12+12+1 = 25 ahead. 
		call	getAD		;[31]
;		nop			;[1]
					; 26 ahead... 

;		movlw	00DH		;delay [44]
		movlw	d'15'		; delay [38] . 26+6= 32 cycles ahead.
		call	delay		;

		call	getAD		; [31] . now we have all 5 xtra conversions.
					; and we are now 1 cycle ahead. 
		bcf	GPIO,0		;[1]
		goto	line_frame_2	;[2]
line_frame_2:	movlw	008H		;delay [29]
		call	delay		;
		call	getAD		;[31]
		movlw	013H		;delay [62]
		call	delay		;
;		nop			;[1]
					; 2 ahead...
		bsf	STATUS,RP0	;this is a nice moment to clear [3]
		clrf	EEADR		;the EEPROM address
		bcf	STATUS,RP0	;
		clrf	OFFSET		; [1]

		goto	line_frame_3	; delay. [2]. we gain 2 nops we deleted earlier. 
line_frame_3:
		return			; [2] + [8] overhead
					; makes a total of [432] = 64us

	; with 28.496 crystal, single instruction is 140.37058 ns long. 
	; so 64us , 64000ns , requires 455.93599 instructions.

	; lot of mess added. but seems fitted 5 extra conversions, removed some nops, added goto
	; and cut delays a bit. should be fine :)

	; THIS PART IS FIXED. 5 extra acquisitions. 


;
; This line routine generates the marker ticks. It is straightforward, but the
; timing is tricky.
line_mark:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]

;		movlw	017H		;delay [74]
		movlw	d'02'		; delay [11] - this is compensating delay. 
		call	delay		;	i do not account it yet. FIXME - damn, fix that mess.

		call	getAD		; [31]
		call	getAD		; [31] [62 total] , 74-62 = 12 cycles ahead. 

;		nop			;[1]		
					; 13 cycles ahead. we need memory badly. 



		call	getAD		;[31]
		movlw	0AH		;10 marker ticks in loop counter [1]


		movwf	CNT_2		;[1]
line_mark_1:	bsf	GPIO,0		;marker tick [2]
		bcf	GPIO,0		;
		movlw	001H		;
		call	delay		;
		goto	line_mark_2	;equals to nops
line_mark_2:	decfsz	CNT_2,1		;
		goto	line_mark_1	;


		nop			;

;		bsf	GPIO,0		;
;		bcf	GPIO,0		;[1]

		goto	saving_memory1	; we save on skipping drawing border...
saving_memory1:	


		call	getAD		;
		call	getAD		;[31]
;		movlw	014H		;delay [65]
;		call	delay		;
		call	getAD		; [31]
		call	getAD		; [31] [62 total] 65-62=3 . 16 cycles ahead. 
;		goto	line_mark_3	;[2]
					; 18 cycles ahead.
		call	getAD		; [31]. 18-31 = 13 cycles short.
					; we need 24 cycles. 
					; so in fact we are still 11 cycles ahead...
					; this is compensated in very first delay .

line_mark_3:	return			; [2] + [8] overhead


	; FIXED. but comments are mess. 
	; but there are 5 extra acquisitions and we are almost done with timings...


					; makes a total of [432] = 64us



;
; This routine generates the vertical sync pulse. Basically is a sync
; pulse with the duration of 3 lines with the normal horizontal sync pulses
; inverted. During the last part of the line we sample the keys. 
; This again is a tricky bit of code. First we sample the input during the
; time that sync is high. If the input is high at that time either the "down"
; key, or the "up" key was pressed. Since we check for the "up" key exclusively
; we first guess it is the "down" key. Next we make sync low and make the 
; video output high (can not be seen since we are outside of the viewing area).
; If the input is high now, it can be either the "select" button, or the "up" 
; key. Since we check for the "up" key later we now guess the "select" key.
; Finally we make both the video as well as the sync output low. If the input 
; still is high it must have been the key which was directly connected
; to +5V ("up"). So it was "up". If non of the keys was pressed the debounce
; counter is initialized to its starting value. If one of the keys was pressed,
; we first look if the debounce counter is 0, if so we simple return and 
; do nothing. If it was not 0 we decrement the debounce counter, if after 
; decrementing it becomes 0 a new key is found and the valid key flag is set.
; The lower nibble of BITS2 is used as the debounce counter. The upper four 
; bits are used as the key flags. (3 bits for 3 keys) and the upper bit for
; valid key found.
line_sync:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]

;		nop			;[1]
					; +1
;		movlw	017H		;delay [74]
;		call	delay		;
		call 	getAD		; [31]
		call	getAD		; [31] [62 total] 74-62 = 13 
					; 14 total ahead.

;		nop			;[1]
					; 15 ahead.


		call	getAD		;[31]
;		movlw	018H		;delay [77]
;		call 	delay		;
		call	getAD		; [31]
		call 	getAD		; [31] [ 62 total] 77-62= 15. 
					; 15+15 = 30 ahead.

					; with 24 cycles needed , it creates need for 54 extra cycles.
					; 31 will be used by 5th getAD. so 54-31 = 23. 					

		call	getAD		;[31]
		movlw	018H		;delay [77]
		call	delay		;

		call	getAD		;[31]

;		movlw	00DH		;delay [44]
		movlw 	d'20' 		; delay [65] . 65-44 = 21 extra cycles. 
					; 23-21 = 2 extra cycles needed
		goto	line_sync_extra	; [2] (and here they are.)
line_sync_extra:
		call	delay		;

		call	getAD		; [31] 


		movlw	000H		;clear W
		btfsc	GPIO,INP_B	;key during sync ?
		movlw	010H		;if so set bit 4
		bcf	GPIO,SYNC_B	;end sync pulse
		bsf	GPIO,VID_B	;VID=1
		btfsc	GPIO,INP_B	;key during vid ?
		movlw	020H		;if so set bit 5
		bcf	GPIO,VID_B	;VID=0 again
		btfsc	GPIO,INP_B	;key anyhow ?
		movlw	040H		;if so set bit 6
		movwf	CNT_1 		;save key code
		movf	CNT_1,1		;copy in itself to set Z if 0
		btfsc	STATUS,Z	;skip if key
		goto	line_sync_1	;no key so return
		movfw	BITS2		;
		andlw	00FH		;label the debounce counter
		btfsc	STATUS,Z	;skip if not 0
		goto	line_sync_2	;
		decfsz	BITS2		;upper nibbl will be 0 !
		goto	line_sync_3	;return if debounce neq 0
		movf	CNT_1,0		;retrieve saved key
		movwf	BITS2		;it is the output code
		bsf	BITS2,7		;set upper bit 
		return
line_sync_1:	movlw	DEBOUNCE	;reload debounce counte
		movwf	BITS2		;
		goto	line_sync_2	;remember a goto replaces 2 nop's
line_sync_2: 	goto	line_sync_3
line_sync_3:	goto	line_sync_4
line_sync_4:	return


	; THIS PART IS FIXED. lot of mess added unfortunatelly. but there are 5 AD aquisitions, 
	; and timing is exact.


;
; This routine displays the text line. This was perhaps one of the most
; complicated parts of the program. First of all, we use a large part of the 
; eeprom memory as a bitmapped display memory. Although writing in the eeprom
; memory is relatively slow (typ. 5ms), reading from it fortunately is very
; fast (next instruction). To gain speed, the characters are first translated
; into a bit map (by routine write_char in the asynchronous main program) and 
; then displayed at high speed from eeprom memory in this routine. The 
; characters are stored in a special way to facilitate high speed reading.
; Remember that at the beginning of the routine the eeprom address byte (EEADR)
; byte is cleared (done in line_frame). At this location we find the first 
; group of bits (byte) to be displayed, namely the upper row of dots from the
; first character. Then EEADR is incremented. At that location we find the 
; bits from the first row of the second character. Again after incrementing 
; EEADR we find the first row of the third character etc. This continues until
; after 7 bytes we have shown the complete first line of the text block. 
; During the second call this this line routine we naturally with 
; building the characters. The second row of the first character can be found
; at the very next memory location (EEADR was not cleared). Note that in this  
; way the bits of the characters are scattered over the eeprom memory in 7
; byte intervals. This is done by the write_char routine. 
line_txt:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]
;		movlw	016H		;delay [71]
;		call	delay		;

		call	getAD		; [31]
		call	getAD		; [31] [62 total]. 71-62=9 cycles ahead. + initial 24, 
					; 24+9 = 33 cycles ahead. 

;		nop			;[1]
					; 34 cycles ahead

		call	getAD		; [31] . 34-31= 3 cycles ahead. 

		call	getAD		;[31]
					;the next part displays the first 
					;group of 4 characters
		movlw	004H		;number of characters [2]
		movwf	CNT_1		;in CNT_1
		bsf	STATUS,RP0	;set SYNC output to input [2]
		bsf	TRISIO,2	;(in bank 1!) and stay in bank 1
line_txt_1:	bsf	STATUS,RP0	;bank 1 [1]
		bsf	EECON1,RD	;initiate read [1]
		rlf	EEDATA,0	;rotate first bit -> w [1]
		incf	EEADR,1		;next byte in EEPROM [1]
		bcf	STATUS,RP0	;bank 0 [1]
		movwf	CNT_2		;w -> CNT_2, bit 1 [2]
		rlf	GPIO,1		;to output		
		rlf	CNT_2,1		;bit 2 [2]
		rlf	GPIO,1		;to output
		rlf	CNT_2,1		;bit 3 [2]
		rlf	GPIO,1		;to output
		rlf	CNT_2,1		;bit 4 [2]
		rlf	GPIO,1		;to output
		rlf	CNT_2,1		;bit 5 [2]
		rlf	GPIO,1		;to output
		bcf	GPIO,0		;output off [1]
		decfsz	CNT_1,1		;next character ? T=[1],F=[2]
		goto	line_txt_1	;[2]
		bsf	STATUS,RP0	;SYNC output is output again [3]
		bcf	TRISIO,2	;
		bcf	STATUS,RP0	;
					;total number of instructions in this
					;part:
					;[2]+[2]+[19]+[19]+[19]+[18]+[3]=[82]
					;is actually 5 to much so we shift
					;get AD one byte
		call	getAD		;[31]
					;the next part displays the timebase 
					;text.
		bsf	STATUS,RP0	;bank 1
		bsf	TRISIO,2	;sync bit becomes input
		bsf	EECON1,RD	;initiate read [1]
		movf	EEDATA,0	;read first character -> w [1]
		movwf	CNT_1		;save it in CNT_1
		incf	EEADR,1		;next byte in EEPROM [1]
		bsf	EECON1,RD	;initiate read [1]
		movf	EEDATA,0	;read second character -> w [1]
		movwf	CNT_2		;save it in CNT_2
		incf	EEADR,1		;next byte in EEPROM [1]
		bcf	STATUS,RP0	;bank 0
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		bcf	GPIO,0		;character off
		nop
		rlf	CNT_2,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_2,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_2,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_2,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_2,1		;display next bit
		rlf	GPIO,1
		bcf	GPIO,0		;character off
		bsf	STATUS,RP0	;bank 1
		bsf	EECON1,RD	;initiate read [1]
		movf	EEDATA,0	;read first character -> w [1]
		movwf	CNT_1		;save it in CNT_1
		incf	EEADR,1		;next byte in EEPROM [1]
		bcf	STATUS,RP0	;bank 0
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		rlf	CNT_1,1		;display next bit
		rlf	GPIO,1
		bcf	GPIO,0		;character off
		bsf	STATUS,RP0	;bank 1
		bcf	TRISIO,2	;sync bit becomes output again
		bcf	STATUS,RP0	;bank 0		

;		movlw	004H		;delay [29]
 		movlw	d'06'		; delay [23] - previous was 17, so 17-23= -6 . 
					; this will be explained below. extra 3 cycles above eaten. 
					; extra 3 wait for us below. 
		call	delay		;
 
					;text part [58]+ delay [17] makes
					;[75]
		call	getAD		;[31]
;		movlw	014H		;delay [65]
;		call	delay		;

		call	getAD		; [31]
		call	getAD		; [31] [62 total] , 65-62 = 3 ahead.
					; + 3 previous ahead, = 6 ahead. 
					; we tweak above delay (previously 17) to add this extra 6. 

;		nop			;[1]
					; this makes inexact timing ! 
					; but we had to conserve memory 

		clrf	OFFSET		; [1]
		return			; [2] + [8] overhead
					; makes a total of [432] = 64us
;
; This line routine takes care of the asynchronous part. On the screen it is
; a black line, but in between the horizontal sync pulse and the second AD
; conversion, the asynchronous part is located. In order to understand how it 
; works we have to realize that the whole synchronous part of the program which
; takes care of the video image on the screen can be considered as one large
; subroutine. This routine is called by the main program after the 
; initialization of the variables, and the entry point to the routine is here
; in this routine (sync_blck). To return to the asynchronous main program
; the working registers etc. are restored and timer 0 is set to generate a new
; interrupt exactly in time to perform the second AD conversion in this line.
; In this way the interrupt routine takes care of the interrupt to itself !
; When this interrupt occurres, the working registers are stored and we continue
; with the synchronous part.
line_asyn:   	bsf	GPIO,SYNC_B	;start sync pulse [1]
		call	getAD		;get sample [31]
		bcf	GPIO,SYNC_B	;stop sync pulse [1]
;		movlw	0C0H		;
		movlw	d'168'		; 216 instead of 192. + 24 instructions.		
		movwf	TMR0		;reset timer0
		movf	SAVE_S,0	;restore status register
		movwf	STATUS	
		movf	SAVE_W,0	;restore working register
		bcf	INTCON,T0IF	;clear interrupt flag
		bsf	INTCON,T0IE	;enable T0 interrupt
		retfie			;return and enable global interrupts.
					;In this part we return to the asynch.
					;main program. On timer0 interrupt
					;we continue this line
sync_blck:	movwf	SAVE_W		;save working register
		movf	STATUS,0	;save STATUS register
		movwf	SAVE_S		
		bcf	STATUS,RP0	;make sure we are in bank 0 [1]
		call	getAD		;[31]

;		movlw	018H		;delay [77]
;		call 	delay		;
		call	getAD
		call	getAD		; 62. 77-62 = 15 extra. 
		
	
		call	getAD		;[31]

;		movlw	018H		;delay [77]
		movlw	d'19'		; delay [62]. 77-62= 15 ahead
		call	delay		;

		call	getAD		;[31]
;		movlw	014H		;delay [65]
;		call	delay		;

		call	getAD
		call	getAD	
					; 62. 65-62= 3 extra + 15 extra = 18 extra. + 15 extra = 33.
		call	getAD		; -31
		goto	sync_blck_two_nops	; -2 . exact.
sync_blck_two_nops:  

		clrf	OFFSET		; [1]
		goto	block_2_ret	; [2] + [8] overhead
					; makes a total of [432] = 64us
;
; This routine takes care of AD conversion, data sampling and trigger function.
; It is called 4 times in every line routine. When the routine is called we
; first see if we are in the progress of taking a sample stream or if the 
; trigger function is disabled (free running). If so the variable timebase is
; decreased. This variable is used to implement the variable timebase. When it
; becomes zero the sample is stored and timebase is reloaded from TB_reload.
; In this way only the TB_reload'th sample is used. This sample is stored in
; the next memory location contained in FSR, and FSR is incremented. 
; Note that the data is stored in the upper 50 bytes from the RAM memory 
; starting from BEGIN_MEM. We continue storing samples until FSR becomes zero.
; This means that the last 20H samples are lost since the upper 32 bytes of the
; RAM memory are not implemented. This was done for speed of simple detection.
; At the moment FSR becomes zero the sample sequence is finished and the sample
; flag is reset. When the routine is called and the sample flag was reset, we 
; are waiting for a trigger condition. We have a trigger condition when the 
; comparator output made a high-to-low transition since the last call to this
; routine (note that the data is connected to the - input of the comparator).
; If the single flag is set, a valid trigger is never generated. Once a valid
; trigger occurred the sample flag is set and sampling starts again. The network
; goto's at the end of the routine take care that all the 5 branches of the
; routine last exactly 32 cycles (note that a goto equals 2 NOPS).
getAD:		bsf	BITS1,NINP	;copy output of comparator in NINP
		btfsc	CMCON,COUT	;and while doing so complement it
		bcf	BITS1,NINP	;
					;
		movf	BITS1,0		;copy BITS1 register in W
		andlw	018H		;label SAMPLE and ALWAYS bits
		btfsc	STATUS,Z	;if both bits are 0 (branches A,B)
		goto	getAD1		;Z will be 1 and we jump to A,B
					;otherwise continue with C,D,E
		decfsz	TIMEBASE,1	;decrement timebase counter
		goto	getAD2		;jump to return if not yet zero
					;continue if zero
		movf	TB_RELOAD,0	;reload value to W
		movwf	TIMEBASE	;and copy it to TIMEBASE
		movf	ADRESH,0	;ADC result high to w
		andlw	0FEH		;make lowest bit 0
		addwf	OFFSET,0	;add OFFSET to w
		movwf	INDF		;store sample
		bsf	ADCON0,GO	;start new conversion
		incf	FSR,1		;increment file select register
		btfss	FSR,7
		goto	getAD8		;goto return
					;so ready with the sampling of line
		bcf	BITS1,SAMPLE	;stop sampling
		movlw	BEGIN_MEM	;reset file select register
		movwf	FSR		;
		goto	getAD10		;end of branch D (longest one)	
					;do branches A,B
getAD1:		movf	BITS1,0		;copy BITS1 register in W
		andlw	07H		;label LAST,NINP,SINGLE bits
		btfss	STATUS,Z	;skip if all 0 (branch A)
		goto	getAD4		;return if no trigger (branch B)
		bsf	BITS1,SAMPLE	;trigger found !
		movlw	BEGIN_MEM	;reset file select register
		movwf	FSR		;
		movlw	001H		;timebase=1
		movwf	TIMEBASE
		goto	getAD7
getAD2:		goto	getAD3		;+14 (incl. return)
getAD3:		nop			;+12 (incl. return)
getAD4:		goto	getAD5		;+11 (incl. return)
getAD5:		goto	getAD6		;+ 9 (incl. return)
getAD6:		goto	getAD7		;+ 7 (incl. return)
getAD7:		nop			;+ 5 (incl. return)
getAD8:		goto	getAD9		;+ 4 (incl. return)
getAD9:		goto	getAD10		
getAD10:	bsf	BITS1,LAST	;copy output of NINP into LAST
		btfsc	BITS1,NINP	;and while doing so complement it
		bcf	BITS1,LAST	;
		return
;
; This routine implements a delay function. A typical call sequence is:
;   movlw   n
;   call    delay
; this will cause a delay (INCLUDING THE MOVELW AND THE CALL) of 5+3n cycles
delay:		movwf	CNT_1
delay1:		decfsz	CNT_1,1
		goto	delay1
		return
;
; This routine is part of the main program.
; This routine writes a byte to the eeprom. The data is in w on call,
; it is returned in w on return. The address is in ADR_EE, this address
; is incremented by 7 on return. Since the synchronous display routines
; access the eeprom, an interrupt to these routines is disabled during the
; write sequence.
; The data in the EEPROM is organized in such a way that the line_txt routine
; which reads the data from the eeprom can read the data sequentially from
; the eeprom and display it on the screen.
; The following table gives the memory location of the characters.
; So for example eeprom memory location 00H contains the row of dots 
; for the first line of the first character. location 2D contains the row of 
; dots for the 7th (and last) line of the 4th character.
; The line_txt routine is called 10 times. During the first 7 calls the 
; characters are displayed (memory locations 00 to 30H). During the next two
; calls to line_txt, memory locations 31H to 3E (containing 00) are displayed
; causing 2 blank lines. Finally during the last call locations 3FH to 45H
; are displayed. These locations contain the cursor.
;
;     00  01  02  03    04  05  06    line 1 char.
;     07  08  09  0A    0B  0C  0D    line 2 char.
;     0E  0F  10  11    12  13  14    line 3 char.
;     15  16  17  18    19  1A  1B    line 4 char.
;     1C  1D  1E  1F    20  21  22    line 5 char.
;     23  24  25  26    27  28  29    line 6 char.
;     2A  2B  2C  2D    2E  2F  30    line 7 char.
;     31  32  33  34    35  36  37    blank
;     38  39  3A  3B    3C  3D  3E    blank
;     3F  40  41  42    43  44  45    cursor
;
write_ee:	bsf	STATUS,RP0	;bank 1
write_ee_1:	btfsc	EECON1,WR	;wait until a previous write is
		goto	write_ee_1	;finished
		bcf	INTCON,GIE	;disable all interrupts
		movwf	EEDATA		;data in w is data to be written
		movf	ADR_EE,0	;ADR_EE is address in eeprom
		movwf	EEADR		;
		bsf	EECON1,WREN	;enable writing in eeprom
		movlw	055H		;initiate write sequence
		movwf	EECON2		;
		movlw	0AAH		;
		movwf	EECON2		;
		bsf	EECON1,WR	;
		bcf	EECON1,WREN	;disable writing
		movlw	007H		;increment write address by 7
		addwf	ADR_EE,1	;write it back to ADR_EE	
		movf	EEDATA,0	;copy data written back in w
		bsf	INTCON,GIE	;enable interrupts again
		bcf	STATUS,RP0	;bank 0
		return
;
; This routine is part of the main program. it writes a character to eeprom.
; On call the character code (the entry address in table char_gen) is in w.
; The location in eeprom is in ADR_EE, on return ADR_EE points to 
; next character. It uses the character ROM table char_gen. Additionally
; the routine clears the next 3 image lines, the lower one of them being
; used for the cursor. The routine checks bit 0 from the character data
; to see if the whole character has been written (see comments for ADR_TAB).
write_char:	movwf	ADR_TAB		;save table address in ADR_TAB
write_char_1:	movf	ADR_TAB,0	;w=ADR_TAB
		call	char_gen	;w=@(w)
		incf	ADR_TAB,1	;ADR_TAB=ADR_TAB+1		
		call	write_ee	;write it in eeprom
		andlw	001H		;
		btfsc	STATUS,Z	;skip if ready	
		goto	write_char_1	;do next row
		movlw	000H		;write three empty rows to accommodate
		call	write_ee	;the cursor
		call	write_ee	;
		call	write_ee	;
		movlw	045H		;adjust ADR_EE to next character
		subwf	ADR_EE,1	;
		return
;
; This routine is part of the main program. It reads a character from eeprom. 
; The address is in ADR_EE on start. On return ADR_EE is incremented by 7 
; (to match the write_ee routine), and the date is in w
read_ee:	bsf	STATUS,RP0	;bank 1
read_ee_1:	btfsc	EECON1,WR	;wait till a previous write is
		goto	read_ee_1	;finished
		bcf	INTCON,GIE	;disable interrupts
		movf	ADR_EE,0	;ADR_EE -> EEADR
		movwf	EEADR		;
		bsf	EECON1,RD	;initiate a read
		movlw	007H		;increment ADR_EE by 7
		addwf	ADR_EE,1	;
		movf	EEDATA,0	;data read in w
		bsf	INTCON,GIE	;enable interrupts again
		bcf	STATUS,RP0	;bank 0
		return
;
; This routine is part of the main program. It handles the placement of the
; cursor and it performs part of the key processing. Upon call the eeprom
; address is in W. The first thing the routine does is placing the cursor
; since write_ee increments ADR_EE with 7 we need to substract 7 from 
; ADR_EE to restore it to its original value. Next the routine waits
; until a key is pressed. If it was the cursor key, carry is set on exit.
select:		movwf	ADR_EE		;save address in ADR_EE
		movlw	0FFH		;code for the cursor
		call	write_ee	;cursor on at location ADR_EE
		movlw	07H		;decrement ADR_EE by 7 to its value
		subwf	ADR_EE,1	;before call
select_1	btfss	BITS2,7		;wait for a key
		goto	select_1	;
		bcf	BITS2,7		;reset key flag
		bcf	STATUS,C	;clear carry
		btfss	BITS2,5		;if not select key return with
		return			;carry not set
		movlw	00H		;remove cursor
		call	write_ee	;
		bsf	STATUS,C	;return with carry set
		return			;
;
; This routine is part of the main program. Depending on the timebase
; setting it writes the correct digits to the eeprom so that it will
; appear so that the timebase setting will appear on the screen.
; On call TB_reload contains the timebase division value (01,02,04,08,10
; 20,40,80 Hex). The routine first searches the correct entry point in 
; table tb_tab. When is has found the entry point it writes the next 
; two bytes from the table to the proper location in the eeprom
set_TB:		movlw	0FDH		;starting with -3, +3 makes 0 !
		movwf	CNT_3		;
set_TB_1	movf	CNT_3,0		;
		addlw	003H		;increment it by 3
		movwf	CNT_3		;
		call	tb_tab		;
		xorwf	TB_RELOAD,0	;
		btfss	STATUS,Z	;if z=1 then w=0 so they are equal
		goto	set_TB_1	;next table entry
		incf	CNT_3,1		;table pointer to first digit
		movlw	004H		;point to first character
		movwf	ADR_EE		;
		movf	CNT_3,0		;
		call	tb_tab		;get the first digit
		call	write_char	;write it to the eeprom
		incf	CNT_3,0		;get the second digit
		call	tb_tab		;write it to the eeprom
		call	write_char	;
		return
;
;
; Here the tables start
		org	0348H
;
; The reference voltage source in combination with the comparator are
; used for the trigger. The reference voltage can be set to voltages
; in the range of 0 - 3.59 V split into a low and high range. I have 
; chosen 8 equidistant values between 0 and 3.59V
;
; no:  Voltage:  range:   VR3-VR0:  VRCON:  Line number:
; 1    0.417     low      2         A2H     16H
; 2    0.870     low      4         A4H     2CH
; 3    1.324     high     1         81H     44H
; 4    1.777     high     4         84H     5AH
; 5    2.230     high     6         86H     72H
; 6    2.683     high     9         89H     88H
; 7    3.137     high     12        8CH     A0H
; 8    3.590     high     15        8FH     B6H
; 
tr_tab:		addwf	PCL,1		;
					;
		retlw	0A2H		;VRCON code for 0.417 V
		retlw	0A4H		;VRCON code for 0.870 V
		retlw	081H		;VRCON code for 1.324 V
		retlw	084H		;VRCON code for 1.777 V
		retlw	086H		;VRCON code for 2.230 V
		retlw	089H		;VRCON code for 2.683 V
		retlw	08CH		;VRCON code for 3.137 V
		retlw	08FH		;VRCON code for 3.590 V
					;
		retlw	016H		;LEVEL_IND code for 0.417 V
		retlw	02CH		;LEVEL_IND code for 0.870 V
		retlw	044H		;LEVEL_IND code for 1.324 V
		retlw	05AH		;LEVEL_IND code for 1.777 V
		retlw	072H		;LEVEL_IND code for 2.230 V
		retlw	088H		;LEVEL_IND code for 2.683 V
		retlw	0A0H		;LEVEL_IND code for 3.137 V
		retlw	0B6H		;LEVEL_IND code for 3.590 V
;
; This is the timebase look-up table. For timebase setting the following
; data is stored in this table:
; 1. The tb_reload value. The tb_reload value determines how many of the 
;    samples are actually used. If tb_reload=01 then every data sample is used
;    if tb_reload=02 every second sample is used, if tb_reload=04 every fourth
;    data sample is used.
; 2. The character code (see next table) for the first time base character
;    on the screen.
; 3. The character code (see next table) for the second time base character
;    on the screen.
; The timebase values on the screen have been rounded to two characters due
; to a shortage of memory. The calculation is as follows: the time between
; two samples is 64/4=16us. There are 50 samples so if every sample is used
; the full scope image takes 50*16=800us=0.8ms.
;
; entry:   tb_reload:  1st char:  2nd char    real     rounded
; 0	   01          00  0.     38  8       800 us   0.8 ms
; 1        02          07  1.     31  6       1.6 ms   1.6 ms
; 2        04          1C  3.     15  2       3.2 ms   3.2 ms
; 3        08          31  6.     23  4       6.4 ms   6.4 ms
; 4        10          0E  1      1C  3      12.8 ms   13  ms 
; 5        20          15  2      2A  5      25.6 ms   25  ms
; 6        40          2A  5      00  0      51.2 ms   50  ms
; 7        80          3F  9      3F  9     102.4 ms   99  ms 
;
tb_tab:		addwf	PCL,1
		retlw	001H		; 0.8
		retlw	000H		;
		retlw	038H		;
		retlw	002H		; 1.6
		retlw	007H		;
		retlw	031H		;
		retlw	004H		; 3.2
		retlw	01CH		;
		retlw	015H		;
		retlw	008H		; 6.4
		retlw	031H		;
		retlw	023H		;
		retlw	010H		; 13
		retlw	00EH		;
		retlw	01CH		;
		retlw	020H		; 25
		retlw	015H		;
		retlw	02AH		;
		retlw	040H		; 50
		retlw	02AH		;
		retlw	000H		;
		retlw	080H		; 99
		retlw	03FH		;
		retlw	03FH		;
;
;
; This is the character look-up table. Note that not all digits are 
; implemented since some were not needed so memory could be saved.
; The following example (digit 0) shows how the characters are coded:
; 
; 76543210 (bit)
;
; .XXX....  070H    Bits 1-7 are actually displayed. For the character itself 
; X...X...  088H    only bits 3-7 are used. Bit 1 is used for the decimal 
; X..XX...  098H    point (if needed). Bit 0 is used to signal the write_char
; X.X.X...  0A8H    routine that the whole character has been read from the 
; XX..X...  0C8H    table.
; X...X...  088H
; .XXX..XX  073H
;
; character set and character code:
; 0. = @(00)    6.    = @(31)   stop      = @(5B)  pos slope = @(85)
; 1. = @(07)    8     = @(38)   normal    = @(62)
; 1  = @(0E)    9     = @(3F)   M         = @(69)
; 2  = @(15)    L     = @(46)   S         = @(70)
; 3. = @(1C)    pause = @(4D)   no trig   = @(77)
; 5  = @(2A)    skip  = @(54)   neg slope = @(7E)
;
;
char_gen:	addwf	PCL,1
					;
		retlw	070H		;"0." @(00)
		retlw	088H		;
		retlw	098H		;
		retlw	0A8H		;
		retlw	0C8H		;
		retlw	088H		;
		retlw	073H		;
					;
		retlw	020H		;"1." @(07)
		retlw	060H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	073H		;
					;
		retlw	020H		;"1" @(0E)
		retlw	060H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	071H		;
					;
		retlw	070H		;"2" @(15)
		retlw	088H		;
		retlw	008H		;
		retlw	010H		;
		retlw	020H		;
		retlw	040H		;
		retlw	0F9H		;
					;
		retlw	0F8H		;"3." @(1C)
		retlw	010H		;
		retlw	020H		;
		retlw	010H		;
		retlw	008H		;
		retlw	088H		;
		retlw	073H		;
					;
		retlw	010H		;"4" @(23)
		retlw	030H		;
		retlw	050H		;
		retlw	090H		;
		retlw	0F8H		;
		retlw	010H		;
		retlw	011H		;
					;
		retlw	0F8H		;"5" @(2A)
		retlw	080H		;
		retlw	0F0H		;
		retlw	008H		;
		retlw	008H		;
		retlw	088H		;
		retlw	071H		;
					;
		retlw	030H		;"6." @(31)
		retlw	040H		;
		retlw	080H		;
		retlw	0F0H		;
		retlw	088H		;
		retlw	088H		;
		retlw	073H		;
					;
		retlw	070H		;"8" @(38)
		retlw	088H		;
		retlw	088H		;
		retlw	070H		;
		retlw	088H		;
		retlw	088H		;
		retlw	071H		;
					;
		retlw	070H		;"9" @(3F)
		retlw	088H		;
		retlw	088H		;
		retlw	078H		;
		retlw	008H		;
		retlw	010H		;
		retlw	061H		;
					;
		retlw	080H		;"L" @(46)
		retlw	080H		;
		retlw	080H		;
		retlw	080H		;
		retlw	080H		;
		retlw	080H		;
		retlw	0F9H		;
					;
		retlw	0D8H		;"pause" @(4D)
		retlw	0D8H		;
		retlw	0D8H		;
		retlw	0D8H		;
		retlw	0D8H		;
		retlw	0D8H		;
		retlw	0D9H		;
					;
		retlw	088H		;"skip" @(54)
		retlw	0C8H		;
		retlw	0E8H		;
		retlw	0F8H		;
		retlw	0E8H		;
		retlw	0C8H		;
		retlw	089H		;
					;
		retlw	000H		;"stop" @(5B)
		retlw	0F8H		;
		retlw	0F8H		;
		retlw	0F8H		;
		retlw	0F8H		;
		retlw	0F8H		;
		retlw	001H		;
					;
		retlw	040H		;"normal" @(62)
		retlw	060H		;
		retlw	070H		;
		retlw	078H		;
		retlw	070H		;
		retlw	060H		;
		retlw	041H		;
					;
		retlw	000H		;"M" @(69)
		retlw	000H		;
		retlw	0D0H		;
		retlw	0A8H		;
		retlw	0A8H		;
		retlw	0A8H		;
		retlw	0A9H		;
					;
		retlw	070H		;"S" @(70)
		retlw	088H		;
		retlw	080H		;
		retlw	070H		;
		retlw	008H		;
		retlw	088H		;
		retlw	071H		;
					;
		retlw	000H		;"always" @(77)
		retlw	000H		;
		retlw	000H		;
		retlw	0F8H		;
		retlw	000H		;
		retlw	000H		;
		retlw	001H		;
					;
		retlw	0E0H		;"neg_slope" @(7E)
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	039H		;
					;
		retlw	038H		;"pos_slope" @(85)
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	020H		;
		retlw	0E1H		;
;
;
        END


